/*
 * flags.h
 *
 *  Created on: Jul 16, 2009
 *      Author: kpcoogan
 */

#ifndef FLAGS_H_
#define FLAGS_H_

/*
 * Instruction flags
 */
#define INSTR_MARKED            (1 <<  0)
#define INSTR_IN_USEDEF         (1 <<  1)  // backward chains from uses to definitions
#define INSTR_IN_DEFUSE         (1 <<  2)  // forward chains from definitions to uses
#define INSTR_IN_BASE_MODULE    (1 <<  3)
#define INSTR_BACKWARD_TAINTED  (1 <<  4)
#define INSTR_FN_ENTRY          (1 <<  5)
#define INSTR_HANDLER_ENTRY     (1 <<  6)
#define INSTR_BBL_START         (1 <<  7)
#define INSTR_BBL_END           (1 <<  8)
#define INSTR_FLAGS_KNOWN       (1 <<  9)
#define INSTR_IGNORED           (1 << 10)
#define INSTR_READS_MEM         (1 << 11)  // instr is a relevant jump instruction
#define INSTR_USED_AS_CALL      (1 << 12)
#define INSTR_MAYBE_DELETED     (1 << 13)  // instr results from simplifying transformations
#define INSTR_READS_GLOBAL_MEM  (1 << 14)  // instr is ud chain target of control flow
#define INSTR_UD_BBL_START      (1 << 15)
#define INSTR_UD_BBL_END        (1 << 16)
#define INSTR_IN_SHARED_MEM     (1 << 17)
#define INSTR_PUSHF             (1 << 18)  // instr is a pushfd instr
#define INSTR_FORWARD_TAINTED   (1 << 19)
#define INSTR_COND_CF           (1 << 20)  // instr is conditional control flow
#define INSTR_TMP0              (1 << 30)  // use as temporary flag, should always be cleared when done!!
#define INSTR_TMP1              (1LLU << 31)  // use as temporary flag, should always be cleared when done!!
/*
 *  additional temporary instruction flags defined by Gen Lu, for label important operands of xchg instruction
 *  labeling address save-use pairs
 *  09/20/2010
 */
#define INSTR_TMP2              (1 << 28)
#define INSTR_TMP3              (1 << 29)

/*
 * Instruction flags for UD simplification
 */
#define INSTR_OP0_CONSTANT          (1 << 21)
#define INSTR_OP1_CONSTANT          (1 << 22)
#define INSTR_DEST_CONSTANT         (1 << 23)
#define INSTR_WRITES_MEM            (1 << 24)
#define INSTR_IS_DELETED            (1 << 25)
#define INSTR_MEM_SIMP              (1 << 26)
#define INSTR_SIMPLIFIED_IGNORED    (1 << 27)
#define INSTR_TEMP_CONDITIONAL      (1LLU << 32)
#define INSTR_JMP_AS_CONDITIONAL    (1LLU << 33)
#define INSTR_UNSIMPLIFIABLE        (1LLU << 34)
#define INSTR_TRNASFORMED			(1LLU << 35)
#define INSTR_BACKWARD_UD			(1LLU << 36)
#define INSTR_READS_TAINTED_MEM		(1LLU << 37)
#define INSTR_USES_FTAINT			(1LLU << 38)
#define INSTR_READS_FLAGSTOREGS		(1LLU << 39)
#define INSTR_DOES_UNPACKING		(1LLU << 40)
#define INSTR_FLAG_FW_TAINED		(1LLU << 41)
#define INSTR_REGDEF_KNOWN			(1LLU << 42)
#define INSTR_TAINT_CLEARED			(1LLU << 43)
#define INSTR_INPUT_SYMBOLIC		(1LLU << 44)
#define INSTR_IS_SYMBOLIC			(1LLU << 45)
#define INSTR_CONDJUMP_TAKEN        (1LLU << 46)



#define INSTR_HAS_FLAG(instr, flag)    (instr ?(bool)(((instr)->flags) & (flag)):false)
#define INSTR_REMOVE_FLAG(instr, flag) (instr ?((instr)->flags) &= ~(flag):false)
#define INSTR_SET_FLAG(instr, flag)    (instr ?((instr)->flags) |= (flag):false)

/*
 * basic block flags
 */
#define BBL_MARKED              (1 <<  0)
#define BBL_IS_DLL              (1 <<  1)
#define BBL_IS_RELEVANT         (1 <<  2)
#define BBL_SPLIT               (1 <<  3)

#define BBL_SET_FLAG(bbl, flag)    (bbl ?((bbl)->flags) |= (flag):false)
#define BBL_HAS_FLAG(bbl, flag)    (bbl ?(bool)(((bbl)->flags) & (flag)):false)
#define BBL_REMOVE_FLAG(bbl, flag) (bbl ?((bbl)->flags) &= ~(flag):false)

/*
 * Operand flags
 */
#define EQOP_MEM_NOT_KNOWN      (1 << 0)
#define EQOP_FLAG_DEP           (1 << 1)

#define EQOP_TEMP0              (1 << 30)
#define EQOP_TEMP1              (1 << 31)

#define EQOP_SET_FLAG(op, flag)    (op ?((op)->flags) |= (flag):false)
#define EQOP_HAS_FLAG(op, flag)    (op ?(bool)(((op)->flags) & (flag)):false)
#define EQOP_REMOVE_FLAG(op, flag) (op ?((op)->flags) &= ~(flag):false)

/*
 * Equation flags
 */
#define EQN_DONT_SIMPLIFY       (1 << 0)
#define EQN_USE                 (1 << 1)    //use equation generated separately
#define EQN_FLAG_DEP            (1 << 2)    //this equation is flag dependent
#define EQN_SET_FLAG(eqn, flag)    (eqn ?((eqn)->flags) |= (flag):false)
#define EQN_HAS_FLAG(eqn, flag)    (eqn ?(bool)(((eqn)->flags) & (flag)):false)
#define EQN_REMOVE_FLAG(eqn, flag) (eqn ?((eqn)->flags) &= ~(flag):false)

/*
 * Expression flags
 */
#define EXP_DONT_SIMPLIFY       (1 << 0)
#define EXP_SIMPLIFIED          (1 << 1)

#define EXP_SET_FLAG(exp, flag)    (exp ?((exp)->flags) |= (flag):false)
#define EXP_HAS_FLAG(exp, flag)    (exp ?(bool)(((exp)->flags) & (flag)):false)
#define EXP_REMOVE_FLAG(exp, flag) (exp ?((exp)->flags) &= ~(flag):false)

/*
 * Simplified Expression flags
 */
#define SE_FLAG_DEP             (1 << 0)
#define SE_IND_FLAG_DEP         (1 << 1)
#define SE_USE                  (1 << 2)

#define SE_SET_FLAG(se, flag)       (se ?((se)->flags) |= (flag): false)
#define SE_HAS_FLAG(se, flag)       (se ?(bool)(((se)->flags) & (flag)): false)
#define SE_REMOVE_FLAG(se, flag)    (se ?((se)->flags) &= ~(flag):false)

/*
 * Tree flags -- none yet
 */

#endif /* FLAGS_H_ */
