/*
 *
 * File: mem_maps.h
 * Author: Babak Yadegari
 */

/*
 * Copyright 2015 Arizona Board of Regents on behalf of the University of Arizona.
 */



#ifndef MEM_MAPS_H_
#define MEM_MAPS_H_

#include "lynx_types.h"


#define MEMORY_LIVE     (char)0xff
#define MEMORY_DEAD     (char)0

#ifdef __cplusplus
extern "C" {
#endif
/*
 * we are keeping track of these structures in iList
 * structure. Every thread in the trace has a separate
 * iList so separate set of these structs.
 */
typedef struct BytetoByteMap{
	uint64_t			startAddrs[8];					// we can calculate the end address adding 8*page_size to the start addr!
	unsigned char*		Constatns;					// one page reserved to tag other 8 pages determining the bytes are constant or not!
	unsigned char*		pages[8];					// the actual address space.
} BytetoByteMap;

typedef struct IndexAddressPair{
	int			index;
	int			addr;				// between 0-7 within a 8-page table!
} IndexAddressPair;

int ClearConstantAndMemoryPages(InstrList *iList, char val);
int ClearConstantPages(InstrList *iList, char val);
bool MemLocIsConstant(InstrList *iList, ADDRESS addr, int numBytes);
int SetAddrAsConstWithValue(InstrList *iList, ADDRESS addr, int numBytes, int val);
int UnsetConstAddr(InstrList *iList, ADDRESS addr, int numBytes);
int GetMemoryLocValue(InstrList *iList, ADDRESS addr, int numBytes);
int SetMemoryLocValue(InstrList *iList, ADDRESS addr, int val);
int GetMemoryLocValue(InstrList *iList, ADDRESS addr, int numBytes);
int SetAddrAsConst(InstrList *iList, ADDRESS addr, int numBytes);
bool PartialMemLocIsConstant(InstrList *iList, ADDRESS addr, int numBytes);
int UnsetConstAddrWithoutMemReset(InstrList *iList, ADDRESS addr, int numBytes);
size_t GetMemoryLocPointer(InstrList *iList, ADDRESS addr);
void SetMemoryLocPointer(InstrList *iList, ADDRESS addr, void *pointer);
int SetMemoryLocValue(InstrList *iList, ADDRESS addr, int val);
int ClearMemoryPages(InstrList *iList, char val);

/******************************************************/

uint64_t DependencyMemGetConstLocSrc(InstrList *iList, ADDRESS addr);
int DependencyMemSetConstWithSrc(InstrList *iList, ADDRESS addr, int numBytes, uint64_t src);


///////////// Taint Anlysis/////////
int UnSetTaintedMemoryLocAsContainsFlags(InstrList *iList, ADDRESS addr, int numBytes);
int TaintMemoryLoc(InstrList *iList, ADDRESS addr, int numBytes, int taint);
int UntaintMemoryLoc(InstrList *iList, ADDRESS addr, int numBytes);
bool MemoryLocIsTainted(InstrList *iList, ADDRESS addr, int numBytes);
int GetMemoryLocTaint(InstrList *iList, ADDRESS addr, int numBytes);
bool TaintedMemoryLocContainsFlags(InstrList *iList, ADDRESS addr, int numBytes);
int SetTaintedMemoryLocAsContainsFlags(InstrList *iList, ADDRESS addr, int numBytes);


//////////// Dead Code Analysis ////

int SetMemoryMap(InstrList *iList, char val);
bool MemoryLocationIsLive(InstrList *iList, ADDRESS addr, int numBytes);
int SetMemoryLocationLive(InstrList *iList, ADDRESS addr, int numBytes);
int SetMemoryLocationDead(InstrList *iList, ADDRESS addr, int numBytes);


/////////// enhanced-taint //////////
int *GetAddrMemBitVars(InstrList *iList, int addr, int size);
int ClearAddrMemBitVars(InstrList *iList, int addr, int size);
int AddAddrMemBitVars(InstrList *iList, int addr, int size, int *bitVars);
int SetMemoryLocTainted(InstrList *iList, ADDRESS addr, int numBytes);

////////////Symbolic Execution///////////
int SetMemoryLocSymbolic(InstrList *iList, ADDRESS addr, int numBytes);
int MakeMemoryLocConcrete(InstrList *iList, ADDRESS addr, int numBytes);
bool MemoryLocIsSymbolic(InstrList *iList, ADDRESS addr, int numBytes);


#ifdef __cplusplus
}
#endif

#endif /* MEM_MAPS_H_ */
