/*
 *
 * File: print.h
 * Code to print out a "reasonable" assembly code program for an
 * instruction trace summary.  This code assumes that the summary
 * list has been sorted by address and that function and exception
 * handler entry points have been marked.
 *
 * Author: Babak Yadegari, Kevin Coogan
 */

/*
 * Copyright 2015 Arizona Board of Regents on behalf of the University of Arizona.
 */



#ifndef PRINT_H_
#define PRINT_H_

/*
 * Execution controls
 */
#define PRINT_DEBUG false

#include "lynx_types.h"

/*
 * Register representation for trace simplification: it's important to keep
 * track of arithmetic operations involving smaller-than-32-bit components
 * of registers.
 */
#define REG_AL   (0x1 << 0)
#define REG_AH   (0x1 << 1)
#define REG_AX   (REG_AL | REG_AH)
#define REG_EAX  ((0x1 << 2) | REG_AX)

#define REG_CL   (0x1 << 3)
#define REG_CH   (0x1 << 4)
#define REG_CX   (REG_CL | REG_CH)
#define REG_ECX  ((0x1 << 5) | REG_CX)

#define REG_DL   (0x1 << 6)
#define REG_DH   (0x1 << 7)
#define REG_DX   (REG_DL | REG_DH)
#define REG_EDX  ((0x1 << 8) | REG_DX)

#define REG_BL   (0x1 << 9)
#define REG_BH   (0x1 << 10)
#define REG_BX   (REG_BL | REG_BH)
#define REG_EBX  ((0x1 << 11) | REG_BX)

#define REG_SP   (0x1 << 12)
#define REG_ESP  ((0x1 << 13) | REG_SP)

#define REG_BP   (0x1 << 14)
#define REG_EBP  ((0x1 << 15) | REG_BP)

#define REG_SI   (0x1 << 16)
#define REG_ESI  ((0x1 << 17) | REG_SI)

#define REG_DI   (0x1 << 18)
#define REG_EDI  ((0x1 << 19) | REG_DI)

void PrintTaintedInstrs(InstrList *list, int start, int n);
void PrintTrace(FILE *f, InstrList *list, int full_list, int printDeadInstrs, int printSigns, uint64_t start, uint64_t end);
void PrintInstrList(InstrList *list);
void PrintFullInstrList(InstrList *list);
void PrintTraceString(InstrList *list);
bool StringIsNotWhitespace(char *str);
char *BitvectorToString(REGISTER_BITVECTOR bv);
char *simplifyRegMask2string(uint32_t rbv);
char *PSWBitVectorToString(PSW_BITVECTOR pv);
char *InstrToString(Instruction *instr);
void PrintPgmInfo(FILE *fp);
void PrintAsmCode(FILE *fp, ArrayList *alist, InstrList *ilist, iEdgeNode *elist);
void PrintMarkedUDChains(InstrList *iList, InstrList *list);
void PrintMemTargets(InstrList *iList, Instruction *iins);
void PrintInstructionSSA_eqn(InstrList *iList, Instruction *iins, unsigned int flags);
void PrintInstruction(FILE *f, InstrList *list, Instruction *next, bool printRegs, bool PrintSigns);
char PrintEdgeType(EdgeType etype);
void PrintControlFlowEdges(ArrayList *edges);
void PrintInstrArrayList(ArrayList *list, ArrayList *edges);
void fPrintiEdge(FILE *fp, iEdge *e);
void PrintiEdge(iEdge *e);
void fPrintEdge(FILE *fp, Edge *e);
void PrintEdge(Edge *e);
void PrintBbl(InstrList *iList, Block *ibbl);
void PrintBBlList();
void PrintMarkedBBLs(InstrList *list);
void PrintBBLSummary(ArrayList *list);
void PrintConstantsList(ArrayList *constants);
void fPrintMemTargets(FILE *f, InstrList *iList, Instruction *iins);
void CFG2DotFile();
char *regname(int udreg);
void PrintSimplifiedTrace(FILE *f, InstrList *list, int full_list, int printDeadInstrs, int printSigns, uint64_t start, uint64_t end);

void PrintSimplifiedInstrList(FILE *f, InstrList *list, int live, int printSigns);
void PrintEntireMultiThreadInstrListInternal(FILE *f, InstrList *list, int full_list, int printDeadInstrs, int tid);

void PrintSimplifiedFullInstrList(FILE *f, InstrList *list, int simple, int printSigns, uint64_t start, uint64_t end);

void PrintOneThread(FILE *f, InstrList *list, int full_list, int ThreadId);

#endif /* PRINT_H_ */
