/*
 *
 * File: simplify.c
 * Purpose: Semantics-preserving simplifications to the code.
 * Author: Babak Yadegari
 */

/*
 * Copyright 2015 Arizona Board of Regents on behalf of the University of Arizona.
 */
#ifndef _SIMPLIFY_H_
#define _SIMPLIFY_H_

#include <inttypes.h>
#include "lynx_types.h"
#include "mem_maps.h"


extern int GlobalMem;
extern int Debug;


#ifdef __cplusplus
extern "C" {
#endif


/*
 * Simplify data structures
 */

// keeps memory defined with the index of instruction
//defining it and its possible content if constant (used in PorpagateConstantsMem)
typedef struct MemDefEl {
    WriteSet *ws;
    int esp;
    int index;
    unsigned int value;
} MemDefEl;


void MarkConstantOperands(InstrList *iList);
void SimplifyConstantsSingleThread(InstrList *iList);
void ForwardTaintPropagation(InstrList *iList, int StartIndex, uint64_t n);
uint64_t MarkUseDefInstrs(InstrList *iList, int first, int last);
int PropagateConstants(InstrList* iList, int first, int last);
int SimplifyGlobalMems(InstrList* iList);
void GlobalMemoryMap(InstrList* iList);
ud_t *ChangeMemOperandToConstant(Instruction *iins, int val, int operand);
int SimplifyGlobalMemsNew(InstrList* iList);
//void MarkImplicitControlDependencies(InstrList *iList);


/*
 * unrelated stuff, should be moved probably!
 */
long long timeval_diff(struct timeval *difference, struct timeval *end_time, struct timeval *start_time);		// calculate time in milliseconds

#ifdef __cplusplus
}
#endif

#endif  /* _SIMPLIFY_H_ */
