/*
 *
 * File: ./src/main.c
 * Author: Babak Yadegari
 */

/*
 * Copyright 2015 Arizona Board of Regents on behalf of the University of Arizona.
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <sys/time.h>
#include <sys/mman.h>


#include "al_helper.h"
#include "instr.h"
#include "lynx_types.h"
#include "print.h"
#include "read.h"
#include "simplify.h"
#include "flowgraph.h"
#include "taint_analysis.h"


 /*********************************************************************
 *                            PROTOTYPES                             *
 *********************************************************************/

/*********************************************************************
 *                              GLOBALS                              *
 *********************************************************************/

//void SimplifyTrace(char *inFile);

ADDRESS entryAddr = 0; // program entry point
char *inFile = NULL; // input trace file
char *baseName = NULL;

bool InstrFileExists = false; // true if using existing instrFile

//ArrayList *iSummary; // summary instruction list
//ArrayList *DLLCallEdges; // list of call edges to DLL routines
//ArrayList *SpecialBblList; // placeholders for DLL routines




extern bool TraceIsMultiThreaded;
extern ArrayList *traceThreads;


void Initialize(){
	InitializeModuleTable(module_names);
	InitializeFunctionNameTable(function_names);
}


int GlobalMem = 1;
int Debug = 0;

int OnlyPrintTrace;
int DetectUnpacking;
int n;
bool RemoveExistinginstrFile = true;
extern int LargestThreadTID;
int print_instrs_in_cfg = false;


#define SIMPLIFY	1

int main(int argc, char *argv[]) {


    Debug = 0;
    handle_rop = false;


    int c;
    int verbose_flag;

    while (1) {
        struct option long_options[] =
        {
                /* These options set a flag. */
                {"verbose", no_argument, &verbose_flag, 1},
                {"brief",   no_argument, &verbose_flag, 0},
                /* These options don't set a flag.
               We distinguish them by their indices. */
                {"debug",     no_argument, 0, 'd'},
                {"global_mem",  no_argument, 0, 'g'},
                {"print_trace",  no_argument, 0, 'p'},
                {"forward-taint",  required_argument, 0, 'f'},
                {"backward-taint", required_argument, 0, 'b'},
                {"print-instrs-in-cfg",    no_argument, &print_instrs_in_cfg, 'i'},
                {"RemoveExistinginstrFile", no_argument, 0, 'r'},
                {"detect_unpacking", no_argument, 0, 'u'},
            {"rop", no_argument, 0, 'o'},
                {0, 0, 0, 0}
        };
        /* getopt_long stores the option index here. */
        int option_index = 0;

        c = getopt_long (argc, argv, "pdrguiob:f:",
                long_options, &option_index);

        /* Detect the end of the options. */
        if (c == -1){
            break;
        }
        switch (c) {
        case 'd':
            Debug = 1;
            break;
        case 'g':
            GlobalMem = 0;
            break;
        case 'p':
            OnlyPrintTrace = 1;				// if on, just prints the trace in "baseName.trace" and exits
            break;
        case 'i':
            print_instrs_in_cfg = true;
            break;
        case 'r':
            RemoveExistinginstrFile = true;
            break;
        case 'u':
        	DetectUnpacking = 1;
        	break;
        case 'o':
            handle_rop = true;
            break;
        case '?':
            if (optopt == 'f') {
                fprintf(stderr, "Option -%c requires a file name.\n", optopt);
            } else if (isprint (optopt)) {
                fprintf(stderr, "Unknown option `-%c'.\n", optopt);
            } else {
                fprintf(stderr, "Unknown option character `\\x%x'.\n", optopt);
            }
            return 1;
        default:
            abort();
            break;
        }
    }


    if (optind < argc)
    {
        Initialize();
        while (optind < argc){
            fprintf(stderr, "Simplifying %s...\n", argv[optind]);

            InstrList *iList = NULL;
            n = 0;

            inFile = argv[optind++];
            iList = InitInstrList(inFile);
            WriteInstrFileHeader(iList, (FILE *)iList->iMMap, true);			// write the header file in case of a crash in the middle!

            ConstantMemoryMap(iList);



            if(OnlyPrintTrace){
                char *trace = malloc(200);
                sprintf(trace, "files/%s.trace", baseName);
                FILE *justtrace = fopen(trace, "w");
                //PrintOneThread(factsimplified, iList, 1, 672);
                PrintSimplifiedFullInstrList(justtrace, iList, 0, 1, 0, iList->lastBaseModuelInstr);
                fclose(justtrace);

                exit(0);

            }



            if(TraceIsMultiThreaded){
                //TraceIsMultiThreaded = false;
                Thread *currThread;
                ThreadRange *currRange;
                int i;
                for (i = 0; i < al_size(traceThreads); i++) {
                    currThread = al_get(traceThreads, i);
                    currRange = al_get(currThread->ranges, 0);
                    if(currThread->tid == LargestThreadTID){
                        iList->currentInstr = currRange->first;
                        currRange = al_get(currThread->ranges, currThread->numRanges - 1);
                        iList->lastInstr = currRange->last;
                        iList->lastBaseModuelInstr = iList->lastInstr;

                        currRange = al_get(currThread->ranges, 0);
                        iList->firstInstr = currRange->first;
                        iList->tid = currThread->tid;

                        //fprintf(stderr, "Thread %d is going to be analyzed starting at: %d!\n", LargestThreadTID, iList->firstInstr);
                        /*iList->numInstrs = currThread->numInstrs;
                         iList-slast = currThread->numInstrs;*/
                        break;
                    }
                }
            }


#if !MAIN_MARKER
            iList->endMarker = iList->last;
#endif
            char *filename;
			FILE *graph;

            // do backward and forward taint analysis
            if(!InstrFileExists){           // run taint analysis once on the trace!
                fprintf(stderr, "Taint analysis...");

                if(handle_rop){
                    TaintAnalysis(iList, TAINT_BITLEVEL, 1);
                } else {
                    TaintAnalysis(iList, TAINT_ENHANCED, 1);
                }

                fprintf(stderr, "done.\n");

            }


            filename = malloc(100);
            sprintf(filename, "files/%s.trace", baseName);
            FILE *simplifiedfile = fopen(filename, "w");     // print simplified trace to a file
            if (TraceIsMultiThreaded) {
                PrintEntireMultiThreadInstrListInternal(simplifiedfile, iList, 1, 0, iList->tid);
            } else {
                PrintSimplifiedTrace(simplifiedfile, iList, 1, 0, 1, 0, iList->numInstrs);
            }
            fclose(simplifiedfile);


			if(iList->endMarker == 0){
				iList->endMarker = -1;
			}


			char *graphname;
            /*CleanUpCFGData();
            ConstructCFG(iList, 0);
            MergeBblsNew();
            char *graphname = malloc(200);
            sprintf(graphname, "files/%s_test.dot", baseName);
            graph = fopen(graphname, "w");
            PrintCFG(graph, 0, iList->startMarker, iList->endMarker+3);

            fclose(graph);
            free(graphname);
            graphname = malloc(200);
            sprintf(graphname, "files/%s_test.bbls", baseName);
            graph = fopen(graphname, "w");
            PrintBblInstrs(graph);
            fclose(graph);
            free(graphname);*/



            // build initial CFG and figure out control dependencies
            fprintf(stderr, "Constructing initial CFG...\n");
            ConstructOroginalCFG(iList);


            filename = malloc(200);
            sprintf(filename, "files/%s.dot", baseName);
            graph = fopen(filename, "w");
            //PrintPlainCFG(graph, iList->startMarker, iList->endMarker);
            PrintCFG(graph, print_instrs_in_cfg, iList->startMarker, iList->endMarker);
            fclose(graph);
            free(filename);

            MergeBblsNew();

            //implicit flow
            DependencyAnalysis(iList);

            if (GlobalMem && !InstrFileExists){         // this should be done before dependency analysis!
                SimplifyGlobalMems(iList);
            }
            fprintf(stderr, "...done\n");


             CleanUpCFGData();

             // simplify trace

            fprintf(stderr, "Simplify...\n");
            SimplifyConstantsSingleThread(iList);
            // write the simplified trace to file
            char *simplifiedname = malloc(100);
            sprintf(simplifiedname, "files/%s.simplified", baseName);
            simplifiedfile = fopen(simplifiedname, "w");     // print simplified trace to a file
            if (TraceIsMultiThreaded) {
                PrintEntireMultiThreadInstrListInternal(simplifiedfile, iList, 0, 0, iList->tid);
            } else {
                PrintSimplifiedTrace(simplifiedfile, iList, 0, 0, 1, 0, iList->numInstrs);
            }

            fclose(simplifiedfile);
            free(simplifiedname);

            // update binary trace format
            WriteInstrFileHeader(iList, (FILE *)iList->iMMap, true);

            // build and write the
            fprintf(stderr, "Constructing simplified CFG!\n");

            CleanUpCFGData();
            ConstructCFG(iList, 0);
            MergeBblsNew();

            graphname = malloc(200);
            sprintf(graphname, "files/%s_simplified.dot", baseName);
            graph = fopen(graphname, "w");
            PrintCFG(graph, print_instrs_in_cfg, iList->startMarker, iList->endMarker+3);

            fclose(graph);
            free(graphname);

            // finishing up
            if(USE_MMAP) {
                //munmap so that dirty pages know to be written out to file
                munmap(iList->iMMap, 0);
            }
            if (!USE_MMAP && STORE_INSTRS_IN_FILE) {
                fclose(iList->iListFile);
                free(iList->instrCache);
            }

            free(iList);
            printf("simplify done\n");

        }


    } else {
        fprintf(stderr, "Usage: %s your_trace_std_trace.txt\n", argv[0]);
        exit(0);
    }


    return (0);
}

