/*
 *  All files in this subdirectory are taken, in whole or in part,
 *  from GDB, the GNU debugger.
 *  Copyright (C) 1988, 89, 91, 93, 94, 95, 96, 97, 1998
 *  Free Software Foundation, Inc.
 *
 *  They have been modified as needed by the LYNX project,
 *  Department of Computer Science,
 *  University of Arizona, 2008-2012.
 *
 *  The GNU General Public license requires the following be included:
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place -- Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 *  x86-registers.h
 */
#ifndef REGISTERS_H
#define REGISTERS_H

enum dis_registers {
    reg_a = 1,                          // EAX     AX      AH      AL
    reg_b = 2,                          // EBX     BX      BH      BL
    reg_c = 3,                          // ECX     CX      CH      CL
    reg_d = 4,                          // EDX     DX      DH      DL
    reg_si = 5,                         // ESI     SI
    reg_di = 6,                         // EDI     DI
    reg_bp = 7,                         // EBP     BP
    reg_sp = 8,                         // ESP     SP

    reg_es = 9,                         // ES
    reg_cs = 10,                        // CS
    reg_ss = 11,                        // SS
    reg_ds = 12,                        // DS
    reg_fs = 13,                        // FS
    reg_gs = 14,                        // GS

    reg_st = 15,                        // Floating point top of stack register
    reg_st0 = 16,                       // Relative to top of st
    reg_st1 = 17,
    reg_st2 = 18,
    reg_st3 = 19,
    reg_st4 = 20,
    reg_st5 = 21,
    reg_st6 = 22,
    reg_st7 = 23,

    reg_all = 24,
    reg_psw = 25,

    reg_cr0 = 26,
    reg_cr2 = 28,
    reg_cr3 = 29,
    reg_cr4 = 30,

    reg_32 = (1 << 5),                  // 32 bit version of reg used
    reg_16 = (1 << 6),                  // 16 bit version of reg used
    reg_8h = (1 << 7),                  // high 8 bit version used
    reg_8l = (1 << 8),                  // low 8 bit version used
    reg_indirect = (1 << 9),
};

#define REG_EAX      (reg_a | reg_32)
#define REG_EBX      (reg_b | reg_32)
#define REG_ECX      (reg_c | reg_32)
#define REG_EDX      (reg_d | reg_32)
#define REG_ESI      (reg_si | reg_32)
#define REG_EDI      (reg_di | reg_32)
#define REG_EBP      (reg_bp | reg_32)
#define REG_ESP      (reg_sp | reg_32)

#define FLAG_AF      1
#define FLAG_CF      2
#define FLAG_OF      3
#define FLAG_PF      4
#define FLAG_SF      5
#define FLAG_ZF      6

#endif                          // REGISTERS_H
