/*
 *  All files in this subdirectory are taken, in whole or in part,
 *  from GDB, the GNU debugger.
 *  Copyright (C) 1988, 89, 91, 93, 94, 95, 96, 97, 1998
 *  Free Software Foundation, Inc.
 *
 *  They have been modified as needed by the LYNX project,
 *  Department of Computer Science,
 *  University of Arizona, 2008-2012.
 *
 *  The GNU General Public license requires the following be included:
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place -- Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef X86_TABLE_H
#define X86_TABLE_H

#include <stdlib.h>

#include "x86-table-defs.h"
#include "x86-registers.h"
#include "x86-classes.h"

/*
 *  The main tables describing the instructions is essentially a copy
 *  of the "Opcode Map" chapter (Appendix A) of the Intel 80386
 *  Programmers Manual.  Usually, there is a capital letter, followed
 *  by a small letter.  The capital letter tell the addressing mode,
 *  and the small letter tells about the operand size.  Refer to
 *  the Intel manual for details.
 */

/*
 * The names OP_E, OP_G, OP_I, OP_J, OP_C, and OP_D have been changed.
 * They have had a 2 appended to each of them in all appropriate places
 * to avoid naming conflicts with udis86 included files.
 *
 * We are using bits of old gdb code along side bits of udis86 code to
 * achieve all the required functionality for instruction disassembly.
 * As a result, there have been some name conflicts that we have had to
 * handle.
 *
 * Kevin Coogan 5/28/12
 */
#define Eb OP_E2, b_mode
#define indirEb OP_indirE, b_mode
#define Gb OP_G2, b_mode
#define Ev OP_E2, v_mode
#define Ed OP_E2, d_mode
#define indirEv OP_indirE, v_mode
#define Ew OP_E2, w_mode
#define Ma OP_E2, v_mode
#define M OP_E2, 0
#define Mp OP_E2, 0
#define Gv OP_G2, v_mode
#define Gw OP_G2, w_mode
#define Rw OP_rm, w_mode
//#define Rd OP_rm, d_mode
#define Rd OP_E2, d_mode
#define Ib OP_I2, b_mode
#define sIb OP_sI, b_mode
#define Iv OP_I2, v_mode
#define Id OP_I2, d_mode
#define Iw OP_I2, w_mode
#define Jb OP_J2, b_mode
#define Jv OP_J2, v_mode
#define Cd OP_C2, d_mode
#define Dd OP_D2, d_mode
#define Td OP_T, d_mode

#define eAX OP_REG, eAX_reg
#define eBX OP_REG, eBX_reg
#define eCX OP_REG, eCX_reg
#define eDX OP_REG, eDX_reg
#define eSP OP_REG, eSP_reg
#define eBP OP_REG, eBP_reg
#define eSI OP_REG, eSI_reg
#define eDI OP_REG, eDI_reg
#define AL OP_REG, al_reg
#define CL OP_REG, cl_reg
#define DL OP_REG, dl_reg
#define BL OP_REG, bl_reg
#define AH OP_REG, ah_reg
#define CH OP_REG, ch_reg
#define DH OP_REG, dh_reg
#define BH OP_REG, bh_reg
#define AX OP_REG, ax_reg
#define DX OP_REG, dx_reg
#define indirDX OP_REG, indir_dx_reg


// MR: Trying to handle SEG registers are regular regs.
#define Sw OP_SEG, w_mode
//#define Sw OP_REG, w_mode
#define Ad OP_DIR, d_mode
#define Ap OP_DIR, 0
#define Av OP_DIR, v_mode
#define Ob OP_OFF, b_mode
#define Ov OP_OFF, v_mode
#define Xb OP_DSSI, b_mode
#define Xv OP_DSSI, v_mode
#define Yb OP_ESDI, b_mode
#define Yv OP_ESDI, v_mode

#define es OP_REG, es_reg
#define ss OP_REG, ss_reg
#define cs OP_REG, cs_reg
#define ds OP_REG, ds_reg
#define fs OP_REG, fs_reg
#define gs OP_REG, gs_reg

#define MX OP_MMX, 0
#define EM OP_EM, v_mode
#define MS OP_MS, b_mode
#define TD OP_3DNow, 0
#define SUF OP_3DNowSuffix, 0
#define PF OP_PREFETCH, b_mode

#define GRP1b TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 0
#define GRP1S TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 1
#define GRP1Ss TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 2
#define GRP2b TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 3
#define GRP2S TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 4
#define GRP2b_one TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 5
#define GRP2S_one TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 6
#define GRP2b_cl TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 7
#define GRP2S_cl TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 8
#define GRP3b TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 9
#define GRP3S TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 10
#define GRP4  TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 11
#define GRP5  TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 12
#define GRP6  TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 13
#define GRP7 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 14
#define GRP8 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 15
#define GRP9 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 16
#define GRP10 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 17
#define GRP11 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 18
#define GRP12 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 19

#define NULL_OPERANDS_3 NULL, 0, NULL, 0, NULL, 0
#define NULL_OPERANDS_2 NULL, 0, NULL, 0
#define NULL_OPERANDS_1 NULL, 0

#define NO_REGISTERS_SOURCE no_reg, no_reg
#define NO_REGISTERS_DESTINATION no_reg, no_reg
#define NO_REGISTERS_ALL NO_REGISTERS_DESTINATION, NO_REGISTERS_SOURCE

#define FLOATCODE 50
#define FLOAT TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, FLOATCODE

#define ST OP_ST, 0
#define STi OP_STi, 0

#define FGRPd9_2 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 0
#define FGRPd9_4 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 1
#define FGRPd9_5 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 2
#define FGRPd9_6 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 3
#define FGRPd9_7 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 4
#define FGRPda_5 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 5
#define FGRPdb_4 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 6
#define FGRPde_3 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 7
#define FGRPdf_4 TYPE_NONE, CLASS_NONE, NULL, COMPLEX, NULL, 8

extern X86op dis386[];
extern X86op dis386_twobyte[];
extern X86op grps[][8];
extern X86op float_mem[];
extern X86op float_reg[][8];
extern X86op fgrps[][8];

#endif                          // X86_TABLE_H
